package JobControl;
use POSIX qw(:sys_wait_h :signal_h);
use subs 'sleep';
use strict;
use vars qw($VERSION);

$VERSION = '0.1';

sub new {
    my $proto = shift;
    my $class = ref($proto) || $proto;

    die "usage: new(\$slots)" unless(@_);
    my ($slots) = @_;
    my $self = { slots => $slots,
                 counter => 0,
                 pids => []
                 };

    bless $self, $class;
}

sub slots {
    my $self = shift;
    $self->{slots} = shift if @_;
    return $self->{slots};
}

sub counter {
    my $self = shift;
    $self->{counter} = shift if @_;
    return $self->{counter};
}

sub fork {
    my $self = shift;
    my $pids = $self->{pids};
    my $slots = $self->{slots};
    while(@$pids >= $slots) {
        wait;
        my $i = 0;
        while($i < @$pids) {
            if(_pid_free($pids->[$i])) {
                splice(@$pids, $i, 1);
                $self->{counter}++;
            } else {
                $i++;
            }
        }
    }
    my $pid = fork();
    push @$pids, $pid;
    return $pid;
}

sub pids {
    my $self = shift;
    return @{$self->{pids}};
}

sub wait {
    my $self = shift;
    my $pids = $self->{pids};

    foreach my $pid (@$pids) {
        waitpid($pid, 0);
    }
    @$pids = ();
}

sub killall {
    my $self = shift;
    my $pids = $self->{pids};

    if(@$pids) {
        kill SIGTERM, @$pids;   # be polite
        sleep @$pids * 0.1;     # test my patience
        kill SIGKILL, @$pids;    # be rude

        $self->wait;            # pick up the pieces
    }
}

#################
### INTERNALS ###
#################

sub sleep {
    my $sec = shift;
    select(undef, undef, undef, $sec);
}

sub _pid_free {
    my $pid = shift;
    my $retval = waitpid($pid, WNOHANG);
    return ($retval == $pid || $retval < 0) ? 1 : 0;
}

1;
__END__
# Below is the stub of documentation for your module. You better edit it!

=head1 NAME

JobControl - Perl extension for blah blah blah

=head1 SYNOPSIS

  use JobControl;
  blah blah blah

=head1 DESCRIPTION

Stub documentation for JobControl was created by h2xs. It looks like the
author of the extension was negligent enough to leave the stub
unedited.

Blah blah blah.

=head1 AUTHOR

Liraz Siri <liraz@bigfoot.com>

=cut
